# CakePHP Glide

[![Build Status](https://img.shields.io/travis/ADmad/cakephp-glide/master.svg?style=flat-square)](https://travis-ci.org/ADmad/cakephp-glide)
[![Coverage Status](https://img.shields.io/codecov/c/github/ADmad/cakephp-glide.svg?style=flat-square)](https://codecov.io/github/ADmad/cakephp-glide)
[![Total Downloads](https://img.shields.io/packagist/dt/ADmad/cakephp-glide.svg?style=flat-square)](https://packagist.org/packages/ADmad/cakephp-glide)
[![License](https://img.shields.io/badge/license-MIT-blue.svg?style=flat-square)](LICENSE.txt)

CakePHP 3.x plugin to help using [Glide](http://glide.thephpleague.com/) image manipulation library.

The plugin consists of a middlware, view helper and a Glide response class.

## Requirements

* CakePHP 3.3+ (For 3.2 and below use 1.x)

## Installation

Install the plugin through composer:

```
composer require admad/cakephp-glide
```

Load the plugin in `config/bootstrap.php` above `DispatcherFactory::add()` calls:

```
Plugin::load('ADmad/Glide');
```

## Usage

### Middleware

In your `Application::middleware()` setup the `GlideMiddleware` which intercepts
requests and serves images generated by Glide. For e.g.:

```php
$middleware->push(new ADmad\Glide\Middleware\GlideMiddleware([
    // Run this filter only for URLs matching specified value. If unset the
    // value of `server.base_url` config mentioned below will be used.
    // http://book.cakephp.org/3.0/en/development/dispatch-filters.html#conditionally-applying-filters
    'scope' => null,

    // Either an instance of League\Glide\Server or config array to be used to
    // create server instance.
    // http://glide.thephpleague.com/1.0/config/setup/
    'server' => [
        // Path or League\Flysystem adapter instance to read images from.
        // http://glide.thephpleague.com/1.0/config/source-and-cache/
        'source' => WWW_ROOT . 'uploads',

        // Path or League\Flysystem adapter instance to write cached images to.
        'cache' => WWW_ROOT . 'cache',

        // URL part to be omitted from source path. Defaults to "/images/"
        // http://glide.thephpleague.com/1.0/config/source-and-cache/#set-a-base-url
        'base_url' => '/images/',

        // Response class for serving images. If unset (default) an instance of
        // \ADmad\Glide\Responses\PsrResponseFactory() will be used.
        // http://glide.thephpleague.com/1.0/config/responses/
        'response' => null,
    ],

    // http://glide.thephpleague.com/1.0/config/security/
    'security' => [
        // Boolean indicating whether secure URLs should be used to prevent URL
        // parameter manipulation. Default false.
        'secureUrls' => false,

        // Signing key used to generate / validate URLs if `secureUrls` is `true`.
        // If unset value of Cake\Utility\Security::salt() will be used.
        'signKey' => null,
    ],

    // If set to `true` exception generated when trying to get image from Glide
    // will be ignored and request will be allowed to proceed. Default false.
    'ignoreException' => false,

    // Cache duration. Default '+1 days'.
    'cacheTime' => '+1 days',

    // Any response headers you may want to set. Default null.
    'headers' => [
        'X-Custom' => 'some-value',
    ]
]));
```

Ideally the above statement should be above the calls to setup middlewares for
`AssetMiddleware` and `RoutingMiddleware` so that `GlideMiddleware` runs before them.

For the example config shown above, for URL like `domain.com/images/user/profile.jpg`
the source image should be under `webroot/uploads/user/profile.jpg`.

__Note__: Make sure the image URL does not directly map to image under webroot.
Otherwise as per CakePHP's default URL rewriting rules the image will be served by
webserver itself and request won't reach your CakePHP app.

### Helper

The provided `GlideHelper` helps creating URLs and image tags for generating
images. You can load the helper in your `AppView::initialize()` as shown in
example below:

```php
public function initialize()
{
    // All option values should match the corresponding options for `GlideFilter`.
    $this->loadHelper('ADmad/Glide.Glide', [
        // Base URL.
        'baseUrl' => '/images/',
        // Whether to generate secure URLs.
        'secureUrls' => false,
        // Signing key to use when generating secure URLs.
        'signKey' => null,
    ]);
}

```

Here are the available methods of `GlideHelper`:

```php
    /**
     * Creates a formatted IMG element.
     *
     * @param string $path Image path.
     * @param array $params Image manipulation parameters.
     * @param array $options Array of HTML attributes and options.
     *   See `$options` argument of `Cake\View\HtmlHelper::image()`.
     * @return string Complete <img> tag.
     */
    GlideHelper::image($path, array $params = [], array $options = [])

    /**
     * URL with query string based on resizing params.
     *
     * @param string $path Image path.
     * @param array $params Image manipulation parameters.
     * @return string Image URL.
     */
    GlideHelper::url($path, array $params = [])
```

The main benefit of using this helper is depending on the value of `secureUrls`
config, the generated URLs will contain a token which will be verified by the
dispatch filter. The prevents modification of query string params.
